# Functions for getting Virtualmin technical support

BEGIN { push(@INC, ".."); };
eval "use WebminCore;";
&init_config();

&foreign_require("at", "at-lib.pl");
&foreign_require("cron", "cron-lib.pl");
&foreign_require("acl", "acl-lib.pl");
&foreign_require("mailboxes", "mailboxes-lib.pl");

$disable_login_cmd = "$module_config_directory/disable-login.pl";
$login_webmin_user = "virtualmin-support";
$virtualmin_username_url = $config{'username_url'} ||
	"https://software.virtualmin.com/cgi-bin/vusername.cgi";
$virtualmin_ticket_url = $config{'ticket_url'} ||
	"https://www.virtualmin.com/bug.php";
$virtualmin_remote_email = $config{'remote_email'} ||
	"remote\@virtualmin.com";
$virtualmin_bug_url = "https://www.virtualmin.com/node/";

# get_virtualmin_ssh_keys()
# Returns a list of SSH authorized keys for Virtualmin tech support, currently
# active for root.
sub get_virtualmin_ssh_keys
{
my @allkeys = &read_authorized_keys(&get_authorized_keys_file());
my @vkeys = grep { $_->{'cmt'} =~ /^Virtualmin:/i } @allkeys;
return @vkeys;
}

# save_virtualmin_ssh_keys(&key, ...)
# Updates the Virtualmin tech support keys to the list provided, which may
# be empty.
sub save_virtualmin_ssh_keys
{
my @vkeys = @_;
my @allkeys = &read_authorized_keys(&get_authorized_keys_file());
my @nonvkeys = grep { $_->{'cmt'} !~ /^Virtualmin:/i } @allkeys;
&write_authorized_keys(&get_authorized_keys_file(), @nonvkeys, @vkeys);
}

# read_authorized_keys(file)
# Returns keys from some file
sub read_authorized_keys
{
my ($file) = @_;
my @rv;
my $lnum = 0;
open(AUTHS, "<".$file);
while(<AUTHS>) {
	s/\r|\n//g;
	my $cmt;
	if (s/#\s*(.*)$//) {
		# Extract comment
		$cmt = $1;
		}
	if (/^((.*)\s+)?(\d+)\s+(\d+)\s+(\d+)\s+(\S+)$/) {
		# SSH1 style line
		my $auth = { 'name' => $6, 'key' => $5, 'exp' => $4,
			     'bits' => $3, 'opts' => $2, 'type' => 1,
			     'line' => $lnum, 'index' => scalar(@rv),
			     'cmt' => $cmt };
		$auth->{'opts'} =~ s/\s+$//;
		push(@rv, $auth);
		}
	elsif (/^((.*)\s+)?([a-z]\S+)\s+(\S+)\s+(\S+)/) {
		# SSH2 style line
		my $auth = { 'name' => $5, 'key' => $4, 'opts' => $2,
			     'keytype' => $3, 'type' => 2,
			     'line' => $lnum, 'index' => scalar(@rv),
			     'cmt' => $cmt };
		$auth->{'opts'} =~ s/\s+$//;
		push(@rv, $auth);
		}
	elsif (/\@/) {
		# Some other line
		my $line = { 'text' => $_, 
			     'line' => $lnum, 'index' => scalar(@rv),
			     'cmt' => $cmt };
		push(@rv, $auth);
		}
	$lnum++;
	}
close(AUTHS);
return @rv;
}

# write_authorized_keys(file, &key, ...)
# Write all provided SSH authorized keys out to a file
sub write_authorized_keys
{
my ($file, @keys) = @_;

# Check if dir exists (such as ~/.ssh)
local $dir = $file;
$dir =~ s/\/[^\/]+$//;
if (!-d $dir) {
	&make_dir($dir, 0700);
	}

# Write out keys
local $existed = -r $file;
&open_tempfile(AUTHS, ">$file");
foreach my $auth (@keys) {
	my $line;
	if ($auth->{'type'} == 2) {
		# SSH 2 format line
		$line = join(" ", $auth->{'opts'} ? ( $auth->{'opts'} ) : ( ),
				  $auth->{'keytype'}, $auth->{'key'},
				  $auth->{'name'} );
		}
	elsif ($auth->{'type'} == 1) {
		# SSH 1 format line
		$line = join(" ", $auth->{'opts'} ? ( $auth->{'opts'} ) : ( ),
				  $auth->{'bits'}, $auth->{'exp'},
				  $auth->{'key'}, $auth->{'name'} );
		}
	elsif ($auth->{'text'}) {
		# Some other line
		$line = $auth->{'text'};
		}
	if ($line) {
		$line .= " # ".$auth->{'cmt'} if ($auth->{'cmt'});
		&print_tempfile(AUTHS, $line,"\n");
		}
	}
&close_tempfile(AUTHS);

# Set sensible permissions
if (!$existed) {
	&set_ownership_permissions(undef, undef, 0600, $file);
	}
}

# get_authorized_keys_file()
# Returns the full path to root's authorized keys file
sub get_authorized_keys_file
{
my @uinfo = getpwnam($config{'user'} || "root");
my $kfile = "$uinfo[7]/.ssh/authorized_keys";
my $kfile2 = "$uinfo[7]/.ssh/authorized_keys2";
return -r $kfile ? $kfile : -r $kfile2 ? $kfile2 : $kfile;
}

# available_virtualmin_ssh_keys()
# Returns a list of keys built into the module for tech support
sub available_virtualmin_ssh_keys
{
return &read_authorized_keys("$module_root_directory/virtualmin_keys");
}

# get_disable_login_job()
# Returns the at job for disabling the remote login, if any
sub get_disable_login_job
{
my @jobs = &at::list_atjobs();
my ($job) = grep { $_->{'realcmd'} =~ /\Q$disable_login_cmd\E/ } @jobs;
return $job;
}

# get_virtualmin_serial()
# Returns the serial number and key for the installed Virtualmi, or undef if
# missing or GPL.
sub get_virtualmin_serial
{
my %serial;
&read_env_file("/etc/virtualmin-license", \%serial);
if ($serial{'SerialNumber'} && $serial{'SerialNumber'} ne 'GPL') {
	return ($serial{'SerialNumber'}, $serial{'LicenseKey'});
	}
return ( );
}

# get_virtualmin_username()
# Returns the 1, the virtualmin.com login and email for the system's serial,
# if possible. If not, returns 0 and an error message.
sub get_virtualmin_username
{
my ($serial, $key) = &get_virtualmin_serial();
return (0, $text{'ticket_epro'}) if (!$serial);
my $url = $virtualmin_username_url.
	  "?serial=".&urlize($serial)."&key=".&urlize($key);
my ($host, $port, $page, $ssl) = &parse_http_url($url);
my ($out, $err);
&http_download($host, $port, $page, \$out, \$err, undef, $ssl);
if ($err) {
	return (0, &text('ticket_efetch', $err));
	}
if ($out =~ /^(BAD|ERR)\s+(.*)/) {
	# Could not get username
	return (0, $1);
	}
elsif ($out =~ /^OK\s*(.*)/) {
	# Got info .. extract
	my %info = map { split(/=/, $_, 2) } split(/\s+/, $1);
	return (1, $info{'username'}, $info{'email'});
	}
else {
	# Unknown error?
	return (0, &text('ticket_eunknown', $out));
	}
}

# post_http_connection(&hostname, port, page, &cgi-params, &out, &err)
# Makes an HTTP post to some URL, sending the given CGI parameters as data.
sub post_http_connection
{
local ($host, $port, $page, $params, $out, $err) = @_;

local $oldproxy = $gconfig{'http_proxy'};	# Proxies mess up connection
$gconfig{'http_proxy'} = '';			# to the IP explicitly
local $h = &make_http_connection($host, $port, 1, "POST", $page);
$gconfig{'http_proxy'} = $oldproxy;
if (!ref($h)) {
	$$err = $h;
	return 0;
	}
&write_http_connection($h, "Host: $host\r\n");
&write_http_connection($h, "User-agent: Webmin\r\n");
&write_http_connection($h, "Content-type: application/x-www-form-urlencoded\r\n");
&write_http_connection($h, "Content-length: ".length($params)."\r\n");
&write_http_connection($h, "\r\n");
&write_http_connection($h, "$params\r\n");

# Read back the results
&complete_http_download($h, $out, $err, undef, 0, $host, $port);
}

# get_external_ip_address()
# Returns the IP address of this system, as seen by other hosts on the Internet.
sub get_external_ip_address
{
local $url = "http://software.virtualmin.com/cgi-bin/ip.cgi";
local ($host, $port, $page, $ssl) = &parse_http_url($url);
local ($out, $error);
&http_download($host, $port, $page, \$out, \$error, undef, $ssl,
	       undef, undef, 5, 0, 1);
$out =~ s/\r|\n//g;
return $error ? undef : $out;
}

1;

